/**
 * Copyright (c) 2013-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * @emails oncall+ui_infra
 * @format
 * @flow
 */

'use strict';

jest.disableAutomock();

jest.mock('generateRandomKey');

const ContentBlock = require('ContentBlock');
const ContentBlockNode = require('ContentBlockNode');
const ContentState = require('ContentState');
const EditorState = require('EditorState');
const Immutable = require('immutable');

const getNextDelimiterBlockKey = require('getNextDelimiterBlockKey');

const {List} = Immutable;

const contentBlocks = [
  new ContentBlock({
    key: 'A',
    text: 'Alpha',
  }),
  new ContentBlock({
    key: 'B',
    text: 'Beta',
  }),
  new ContentBlock({
    key: 'C',
    text: 'Charlie',
  }),
];

const contentBlockNodes = [
  new ContentBlockNode({
    key: 'A',
    text: 'Alpha',
    nextSibling: 'B',
  }),
  new ContentBlockNode({
    key: 'B',
    text: '',
    children: List(['C']),
    nextSibling: 'D',
    prevSibling: 'A',
  }),
  new ContentBlockNode({
    key: 'C',
    parent: 'B',
    text: 'Charlie',
  }),
  new ContentBlockNode({
    key: 'D',
    text: '',
    prevSibling: 'B',
    children: List(['E', 'F']),
  }),
  new ContentBlockNode({
    key: 'E',
    parent: 'D',
    nextSibling: 'F',
    text: 'Elephant',
  }),
  new ContentBlockNode({
    key: 'F',
    parent: 'D',
    prevSibling: 'E',
    text: 'Fire',
  }),
];

const assertGetNextDelimiterBlockKey = (
  targetBlockKey,
  blocksArray = contentBlockNodes,
) => {
  const editor = EditorState.createWithContent(
    ContentState.createFromBlockArray(blocksArray),
  );
  const contentState = editor.getCurrentContent();
  const targetBlock = contentState.getBlockForKey(targetBlockKey);

  expect(
    getNextDelimiterBlockKey(targetBlock, contentState.getBlockMap()),
  ).toMatchSnapshot();
};

test('must return null when block is ContentBlock', () => {
  assertGetNextDelimiterBlockKey('A', contentBlocks);
});

test('must return null when only blocks after it are its own descendants', () => {
  assertGetNextDelimiterBlockKey('D');
});

test('must return null when block is the last block', () => {
  assertGetNextDelimiterBlockKey('F');
});

test('must find its next sibling when has siblings', () => {
  assertGetNextDelimiterBlockKey('E');
});

test('must find its next delimiter when block does not have siblings', () => {
  assertGetNextDelimiterBlockKey('C');
});
